﻿using System;
using System.Drawing;
using System.Threading.Tasks;

using DevExpress.XtraBars;
using DevExpress.XtraBars.Ribbon;
using DevExpress.XtraEditors;
using DevExpress.XtraSplashScreen;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : RibbonForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 취소 버튼 일반 이미지
        /// </summary>
        private Image cancelButtonNormalImage;

        /// <summary>
        /// 취소 버튼 활성 이미지
        /// </summary>
        private Image cancelButtonActiveImage;

        /// <summary>
        /// 오버레이 텍스트 페인터
        /// </summary>
        private OverlayTextPainter overlayTextPainter;

        /// <summary>
        /// 오버레이 이미지 페인터
        /// </summary>
        private OverlayImagePainter overlayImagePainter;

        /// <summary>
        /// 작업 헬퍼
        /// </summary>
        private TaskHelper taskHelper;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.cancelButtonNormalImage = GetCancelButtonNormalImage();
            this.cancelButtonActiveImage = GetCancelButtonActiveImage();

            this.overlayImagePainter = new OverlayImagePainter(this.cancelButtonNormalImage, this.cancelButtonActiveImage, ProcessCancelButtonClick);
            this.overlayTextPainter  = new OverlayTextPainter();

            this.runTaskBarButtonItem.ItemClick += runTaskButton_ItemClick;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 작업 실행 버튼 항목 클릭시 처리하기 - runTaskButton_ItemClick(sender, e)

        /// <summary>
        /// 작업 실행 버튼 항목 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void runTaskButton_ItemClick(object sender, ItemClickEventArgs e)
        {
            this.runTaskBarButtonItem.Enabled = false;

            string taskResult;

            IOverlaySplashScreenHandle overlayHandle = SplashScreenManager.ShowOverlayForm
            (
                this.contentPanel,
                customPainter : new OverlayWindowCompositePainter(this.overlayTextPainter, this.overlayImagePainter)
            );

            try
            {
                taskResult = await RunTask();
            }
            finally
            {
                SplashScreenManager.CloseOverlayForm(overlayHandle);

                this.runTaskBarButtonItem.Enabled = true;
            }

            XtraMessageBox.Show(this, taskResult, "작업 결과");
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 취소 버튼 일반 이미지 구하기 - GetCancelButtonNormalImage()

        /// <summary>
        /// 취소 버튼 일반 이미지 구하기
        /// </summary>
        /// <returns>취소 버튼 이미지</returns>
        private Image GetCancelButtonNormalImage()
        {
            return ImageHelper.CreateImage(Properties.Resources.CancelNormal);
        }

        #endregion
        #region 취소 버튼 활성 이미지 구하기 - GetCancelButtonActiveImage()

        /// <summary>
        /// 취소 버튼 활성 이미지 구하기
        /// </summary>
        /// <returns>활성 취소 버튼 이미지</returns>
        private Image GetCancelButtonActiveImage()
        {
            return ImageHelper.CreateImage(Properties.Resources.CancelActive);
        }

        #endregion

        #region 취소 버튼 클릭시 처리하기 - ProcessCancelButtonClick()

        /// <summary>
        /// 취소 버튼 클릭시 처리하기
        /// </summary>
        private void ProcessCancelButtonClick()
        {
            if(this.taskHelper != null)
            {
                this.taskHelper.Cancel();
            }
        }

        #endregion
        #region 진행 상태 변경시 처리하기 - ProcessProgressChanged(value)

        /// <summary>
        /// 진행 상태 변경시 처리하기
        /// </summary>
        /// <param name="value">값</param>
        private void ProcessProgressChanged(int value)
        {
            this.overlayTextPainter.Text = value.ToString() + "%";
        }

        #endregion

        #region 작업 실행하기 - RunTask()

        /// <summary>
        /// 작업 실행하기
        /// </summary>
        /// <returns>작업 실행 태스크</returns>
        private async Task<string> RunTask()
        {
            string taskResult;

            this.taskHelper = new TaskHelper();

            try
            {
                taskResult = await this.taskHelper.Start(new Progress<int>(ProcessProgressChanged));
            }
            catch(OperationCanceledException)
            {
                taskResult = "작업이 취소되었습니다.";
            }
            finally
            {
                this.taskHelper.Dispose();

                this.taskHelper = null;
            }

            return taskResult;
        }

        #endregion
    }
}