﻿using System;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 작업 헬퍼
    /// </summary>
    public class TaskHelper : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 취소 토큰 소스
        /// </summary>
        private CancellationTokenSource cancellationTokenSource;

        /// <summary>
        /// 취소 토큰
        /// </summary>
        private CancellationToken cancellationToken;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TaskHelper()

        /// <summary>
        /// 생성자
        /// </summary>
        public TaskHelper()
        {
            this.cancellationTokenSource = new CancellationTokenSource();

            this.cancellationToken = cancellationTokenSource.Token;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start(progress)

        /// <summary>
        /// 시작하기
        /// </summary>
        /// <param name="progress">진행 인터페이스</param>
        /// <returns>작업 태스크</returns>
        public Task<string> Start(IProgress<int> progress)
        {
            return StartInternal(progress);
        }

        #endregion
        #region 취소하기 - Cancel()

        /// <summary>
        /// 취소하기
        /// </summary>
        public void Cancel()
        {
            this.cancellationTokenSource.Cancel();
        }

        #endregion

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">해제 여부</param>
        protected virtual void Dispose(bool disposing)
        {
            if(disposing)
            {
                if(this.cancellationTokenSource != null)
                {
                    this.cancellationTokenSource.Dispose();
                }
            }

            this.cancellationTokenSource = null;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 계산하기 - Calculate(progress)

        /// <summary>
        /// 계산하기
        /// </summary>
        /// <param name="progress">진행 인터페이스</param>
        /// <returns>계산 결과</returns>
        private string Calculate(IProgress<int> progress)
        {
            for(int i = 0; i <= 100; i++)
            {
                Thread.Sleep(100);

                this.cancellationToken.ThrowIfCancellationRequested();

                if(progress != null)
                {
                    progress.Report(i);
                }
            }

            return "계산 결과";
        }

        #endregion
        #region 시작하기 (내부용) - StartInternal(progress)

        /// <summary>
        /// 시작하기 (내부용)
        /// </summary>
        /// <param name="progress">작업 인터페이스</param>
        /// <returns>작업 태스크</returns>
        private Task<string> StartInternal(IProgress<int> progress)
        {
            return Task.Run(() => Calculate(progress), this.cancellationToken);
        }

        #endregion
    }
}